<?
/************************************************************************************************************
	ldap_extend.inc.php
	
	Holds functions revolving around samba and posix account manipulation

***********************************************************************************************************/


//add samba functionality to an account by adding the base samba entries
function addSamba($accountInfo) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	$arr = array();
	$dn = $accountInfo["dn"];

	$password = md5(rand());
	
	//make a default password.  The user won't be able to login until the password is reset
	$ntpwd = `mkntpwd "$password"`;		//mkntpwd must be in apache's path
	$pwd = @explode(":",$ntpwd);		//printfs "ENCLMPWD:ENCNTPWD"
	if (is_array($pwd)) {
		$arr["sambaNTPassword"] = $pwd[1];
		$arr["sambaLMPassword"] = $pwd[0];
	}

	$arr["sambaAcctFlags"] = "[U ]";

	//since no sid is passed with the user info, use a new random one
	$arr["sambaSID"] = "S-1-5-21-0-0-1-".$accountInfo["id"];

	//setup our objectclass array
	$accountInfo["objectClass"][] = "sambaSamAccount";
	$arr["objectClass"] = array_values($accountInfo["objectClass"]);

	if (ldap_modify($ds,"$dn",$arr)) return true;
	else return false;
       
}

//set samba parameters for this account
function updateSamba($option) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	extract($option);
	
	if (!$login) {
		$result["errorMessage"] = "You must enter a login to modify the samba settings.";
		return $result;
	}

	if (!$search_base) $search_base = SEARCH_BASE;
	
	$dn = "uid=".$login.",".$search_base;

	$arr = array();
	if ($sambaHomePath) $arr["sambaHomePath"] = $sambaHomePath;
	if ($sambaHomeDrive) $arr["sambaHomeDrive"] = $sambaHomeDrive;
	if ($sambaProfilePath) $arr["sambaProfilePath"] = $sambaProfilePath;
	//if ($sambaLogonScript) $arr["sambaLogonScript"] = $sambaLogonScript;
	if ($sambaDomainName) $arr["sambaDomainName"] = $sambaDomainName;
	if ($sambaSID) $arr["sambaSID"] = $sambaSID;
	if ($sambaPrimaryGroupSID) $arr["sambaPrimaryGroupSID"] = $sambaPrimaryGroupSID;

	if (ldap_modify($ds,"$dn", $arr)) $result["successMessage"] = "Account updated successfully";
	else $result["errorMessage"] = "Account updated failed";
	
	return $result;

}

//remove samba functionality from the account
function removeSamba($accountInfo) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	$arr = array();
	$dn = $accountInfo["dn"];

	$arr["sambaNTPassword"] = array();
	$arr["sambaLMPassword"] = array();
	$arr["sambaSID"] = array();
	$arr["sambaAcctFlags"] = array();
	$arr["objectClass"] = "sambaSamAccount";
	$arr["sambaHomePath"] = array();
	$arr["sambaHomeDrive"] = array();
	$arr["sambaProfilePath"] = array();
	//$arr["sambaLogonScript"] = array();
	$arr["sambaDomainName"] = array();
	$arr["sambaPrimaryGroupSID"] = array();
	
	if (ldap_mod_del($ds,$dn,$arr)) return true;
	else return false;
       
}


function addShadow($accountInfo) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	$arr = array();
	$dn = $accountInfo["dn"];

	$accountInfo["objectClass"][] = "shadowAccount";
	$arr["objectClass"] = array_values($accountInfo["objectClass"]);

	//these need real values
	$arr["shadowLastChange"]="12129";
	$arr["shadowMax"]="99999";
	$arr["shadowWarning"]="7";
	$arr["loginShell"]="/bin/bash";

	if (ldap_modify($ds,"$dn",$arr)) return true;
	else return false;

	//we also need to add this account to the admin group to allow login
       
}

function removeShadow($accountInfo) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	$arr = array();
	$dn = $accountInfo["dn"];
	
	$arr["shadowLastChange"]=array();
	$arr["shadowMax"]=array();
	$arr["shadowWarning"]=array();
	$arr["loginShell"]=array();
	$arr["objectClass"] = "shadowAccount";

	if (ldap_mod_del($ds,$dn,$arr)) return true;
	else return false;

	//we also need to remove this account to the admin group to disallow login
       
}


function updateSambaPassword($accountId,$password) {

        $ds = ldap_connect(LDAP_SERVER,LDAP_PORT);
	ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, LDAP_PROTOCOL);
	$r = ldap_bind($ds,BIND_DN,BIND_PASSWORD);

	$arr = array();
	$result = array();
	
	if ($accountId==NULL) {
		$result["errorMessage"] = "The account id must be passed to update the account";
		return $result;
	}

	if (!$password) {
		$result["errorMessage"] = "You must specify a password";
		return $result;
	}

	//get our current account info
	$accountInfo = returnAccountInfo(null,$accountId,null);
	$login = $accountInfo["login"];
	
	//since we go by account id we should be able to just yank the search base from here
	$searchBase = $accountInfo["searchBase"];

	$dn = "uid=".$login.",".$searchBase;

	//do something special if we are using samba
	if ($accountInfo["sambaEnable"]) {
		echo "yes";
		$ntpwd = `mkntpwd "$password"`;		//mkntpwd must be in apache's path
		$pwd = @explode(":",$ntpwd);		//printfs "ENCLMPWD:ENCNTPWD"
		if (is_array($pwd)) {
			$arr["sambaNTPassword"] = $pwd[1];
			$arr["sambaLMPassword"] = $pwd[0];
		}

		if (ldap_modify($ds,"$dn", $arr)) $result["successMessage"] = "Account updated successfully";
		else $result["errorMessage"] = "Account updated failed";
	
		return $result;
	

	}
	
	return true;
	
}
